export declare class ClientLibrary {
    private _backendApi;
    /**
     * Construct the Client Library instance. Optionally
     * pass the backend URL to initialize directly.
     * @param newBackendUrl
     */
    constructor(newBackendUrl?: string);
    /**
     * Initialize the web3Provider by providing the URL
     * to the backend.
     * @param newBackendUrl
     * @throws InvalidUrlError
     */
    initialize(newBackendUrl: string): void;
    /**
     * Authenticate the user by asking them to sign
     * a message. The web3Provider argument is used
     * to sign the authentication message.
     * This returns a JWT token which should be used
     * for subsequent calls.
     * @param account
     * @param web3Provider
     * @returns JWT Token
     */
    authenticateUser(account: string, web3Provider: any): Promise<string>;
    /**
     * This handles the sequence of interactions to
     * complete the Commit action (with backend and
     * with contracts via the contracts SDK library).
     * @param web3Provider
     * @param voucherSetId
     * @param jwt
     * @returns Success status of commit
     */
    commitToBuy(web3Provider: any, voucherSetId: string, jwt: any): string;
    /**
     * Compose the domain object.
     * @param chainId
     * @private
     * @returns Domain object
     */
    private static getDomain;
    /**
     * Composes the domain definition object.
     * @private
     * @returns Domain definition object
     */
    private static getEIP712Domain;
}
//# sourceMappingURL=index.d.ts.map