"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ClientLibrary = void 0;
var axios_1 = __importDefault(require("axios"));
var utils_1 = require("ethers/lib/utils");
var backendApi_1 = require("./backend/backendApi");
var ethers_1 = require("./helpers/ethers");
var validityHelper_1 = require("./helpers/validityHelper");
var errors_1 = require("./errors");
var ClientLibrary = /** @class */ (function () {
    /**
     * Construct the Client Library instance. Optionally
     * pass the backend URL to initialize directly.
     * @param newBackendUrl
     */
    function ClientLibrary(newBackendUrl) {
        if (newBackendUrl) {
            this.initialize(newBackendUrl);
        }
    }
    /**
     * Initialize the web3Provider by providing the URL
     * to the backend.
     * @param newBackendUrl
     * @throws InvalidUrlError
     */
    ClientLibrary.prototype.initialize = function (newBackendUrl) {
        if (validityHelper_1.isValidUrl(newBackendUrl)) {
            this._backendApi = new backendApi_1.BackendApi(axios_1.default.create({
                baseURL: newBackendUrl
            }));
        }
        else {
            throw new errors_1.InvalidUrlError(newBackendUrl);
        }
    };
    /**
     * Authenticate the user by asking them to sign
     * a message. The web3Provider argument is used
     * to sign the authentication message.
     * This returns a JWT token which should be used
     * for subsequent calls.
     * @param account
     * @param web3Provider
     * @returns JWT Token
     */
    ClientLibrary.prototype.authenticateUser = function (account, web3Provider) {
        return __awaiter(this, void 0, void 0, function () {
            var nonce, EIP712Domain, domain, AuthSignature, data, isSmartWalletAccount, signatureLike, signature;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this._backendApi.generateNonce(account)];
                    case 1:
                        nonce = _a.sent();
                        EIP712Domain = ClientLibrary.getEIP712Domain();
                        domain = ClientLibrary.getDomain(web3Provider.network.chainId);
                        AuthSignature = [{ name: "value", type: "string" }];
                        data = JSON.stringify({
                            types: {
                                EIP712Domain: EIP712Domain,
                                AuthSignature: AuthSignature,
                            },
                            domain: domain,
                            primaryType: "AuthSignature",
                            message: {
                                value: "Authentication message: " + nonce
                            },
                        });
                        isSmartWalletAccount = ethers_1.isSmartWallet(web3Provider);
                        return [4 /*yield*/, web3Provider.send("eth_signTypedData_v4", [account, data])];
                    case 2:
                        signatureLike = _a.sent();
                        return [4 /*yield*/, utils_1.splitSignature(signatureLike)];
                    case 3:
                        signature = _a.sent();
                        return [4 /*yield*/, this._backendApi.verifySignature(account, isSmartWalletAccount, domain, { AuthSignature: AuthSignature }, signature)];
                    case 4: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    /**
     * This handles the sequence of interactions to
     * complete the Commit action (with backend and
     * with contracts via the contracts SDK library).
     * @param web3Provider
     * @param voucherSetId
     * @param jwt
     * @returns Success status of commit
     */
    ClientLibrary.prototype.commitToBuy = function (web3Provider, voucherSetId, jwt) { return ""; };
    /**
     * Compose the domain object.
     * @param chainId
     * @private
     * @returns Domain object
     */
    ClientLibrary.getDomain = function (chainId) {
        return {
            name: "Boson Protocol",
            version: "1",
            chainId: chainId,
            verifyingContract: "0xCcCCccccCCCCcCCCCCCcCcCccCcCCCcCcccccccC",
        };
    };
    /**
     * Composes the domain definition object.
     * @private
     * @returns Domain definition object
     */
    ClientLibrary.getEIP712Domain = function () {
        return [
            { name: "name", type: "string" },
            { name: "version", type: "string" },
            { name: "chainId", type: "uint256" },
            { name: "verifyingContract", type: "address" },
        ];
    };
    return ClientLibrary;
}());
exports.ClientLibrary = ClientLibrary;
//# sourceMappingURL=index.js.map